import { SourceChainTransferData, EvmSigner } from '@moonbeam-network/xcm-sdk';
import * as _moonbeam_network_xcm_types from '@moonbeam-network/xcm-types';
import { AssetAmount, AnyChain, EvmChain, EvmParachain, Ecosystem, AnyAsset } from '@moonbeam-network/xcm-types';
import { Signer } from '@polkadot/api/types';
import { IKeyringPair, ISubmittableResult } from '@polkadot/types/types';
import { TokenTransfer } from '@wormhole-foundation/sdk-connect';
import { WalletClient } from 'viem';
import { ConfigService } from '@moonbeam-network/xcm-config';

interface Signers {
    evmSigner?: WalletClient;
    polkadotSigner?: Signer | IKeyringPair;
}
interface TransferData {
    destination: DestinationTransferData;
    getEstimate(amount: number | string): AssetAmount;
    isAutomaticPossible: boolean;
    max: AssetAmount;
    min: AssetAmount;
    moonChain: MoonChainTransferData;
    source: SourceTransferData;
    transfer(amount: bigint | number | string, isAutomatic: boolean, signers: Signers, statusCallback?: (params: ISubmittableResult) => void, sendOnlyRemoteExecution?: boolean): Promise<string[]>;
}
interface SourceTransferData extends SourceChainTransferData {
    destinationFeeBalance: AssetAmount;
    moonChainFeeBalance?: AssetAmount;
    relayerFee?: AssetAmount;
    feeBalance: AssetAmount;
    max: AssetAmount;
}
interface DestinationTransferData extends ChainTransferData {
}
type MoonChainTransferData = Omit<ChainTransferData, 'min'> & {
    address: string;
    feeBalance: AssetAmount;
};
interface ChainTransferData {
    chain: AnyChain;
    balance: AssetAmount;
    fee: AssetAmount;
    min: AssetAmount;
}
interface ExecuteTransferData {
    vaa: TokenTransfer.VAA;
    tokenTransfer: TokenTransfer;
    executeTransfer(signer: EvmSigner): Promise<string>;
}

interface WormholeExecuteTransferParams {
    txId: string;
    chain: EvmChain | EvmParachain;
}

interface MrlOptions {
    configService?: ConfigService;
    ecosystem?: Ecosystem;
}
declare function Mrl(options?: MrlOptions): {
    sources: AnyChain[];
    setSource(source: string | AnyChain): {
        destinations: AnyChain[];
        setDestination(destination: string | AnyChain): {
            assets: _moonbeam_network_xcm_types.Asset[];
            setAsset(asset: string | AnyAsset): {
                setIsAutomatic(isAutomatic: boolean): {
                    setAddresses({ sourceAddress, destinationAddress, }: {
                        sourceAddress: string;
                        destinationAddress: string;
                    }): Promise<TransferData>;
                };
            };
        };
    };
    getExecuteTransferData({ txId, chain }: WormholeExecuteTransferParams): Promise<ExecuteTransferData>;
};

export { type ChainTransferData, type DestinationTransferData, type ExecuteTransferData, type MoonChainTransferData, Mrl, type MrlOptions, type Signers, type SourceTransferData, type TransferData };
